from pms import user_controller
import pathlib
from pms import db_functions

username = "ahmad"
password = "123"


def pre_set_db(module=user_controller, db_manager=db_functions):
    path = pathlib.Path.joinpath(pathlib.Path(__file__).parent, "sql_testing.db")
    module.db_path = path
    try:
        db_manager.delete_db(path)
    except FileNotFoundError:
        pass


def test_user_signup():
    pre_set_db()
    user = user_controller.signup_user("ahmad", "123")
    assert user is not None


def test_user_signup_already_in():
    pre_set_db()
    user1 = user_controller.signup_user(username, password)
    assert user1 is not None
    user2 = user_controller.signup_user(username, password)
    assert user2 is None


def test_login_valid_user(capsys):
    pre_set_db()
    user_controller.signup_user(username, password)
    user_controller.login_user(username, password)
    captured = capsys.readouterr()
    assert "user logged in" in captured.out


def test_login_user_invalid_username():
    pre_set_db()
    user_controller.signup_user(username, password)
    user = user_controller.login_user(username + "1", password)
    assert user is None


def test_login_user_invalid_password():
    pre_set_db()
    user_controller.signup_user(username, password)
    try:
        user = user_controller.login_user(username, password + "1")
        assert user is None
    except Exception as e:
        print(e)
        assert False


def test_double_login():
    pre_set_db()
    user_controller.signup_user(username, password)
    user_controller.signup_user(username + "1", password)
    user_controller.login_user(username, password)
    user = user_controller.login_user(username + "1", password)
    assert user is None


def test_logout(capsys):
    pre_set_db()
    user_controller.signup_user(username, password)
    user_controller.login_user(username, password)
    user_controller.logout()
    captured = capsys.readouterr()
    assert "user logged out" in captured.out


def test_logout_without_login(capsys):
    pre_set_db()
    user_controller.signup_user(username, password)
    user_controller.logout()
    captured = capsys.readouterr()
    assert "user not login!" in captured.out


def test_logout_twice(capsys):
    pre_set_db()
    user_controller.signup_user(username, password)
    user_controller.logout()
    user_controller.logout()
    captured = capsys.readouterr()
    assert "user not login!" in captured.out



import argparse
from pms import user_controller
from pms import password_controller
from pms import utility_functions


def main():
    global_parser = argparse.ArgumentParser()
    subparsers = global_parser.add_subparsers(
        dest="subparser_name",
        title="subcommands",
        help="password management system"
    )

    generate_password_parser = subparsers.add_parser("generate", help="generate a password")
    generate_password_parser.add_argument("-l", "--long", action="store", type=int, default=16)
    generate_password_parser.add_argument("-q", "--query", nargs="+")

    evaluate_password_parser = subparsers.add_parser("evaluate", help="evaluate the strength of a password")
    evaluate_password_parser.add_argument("password")

    signup_parser = subparsers.add_parser("signup", help="signup user")
    signup_parser.add_argument("username")
    signup_parser.add_argument("password")

    login_parser = subparsers.add_parser("login", help="login")
    login_parser.add_argument("username")
    login_parser.add_argument("password")

    logout_parser = subparsers.add_parser("logout", help="logout")

    show_user_parser = subparsers.add_parser("user", help="show logged user if found")

    add_password_parser = subparsers.add_parser("add", help="add password")
    add_password_parser.add_argument("new_password")
    add_password_parser.add_argument("name")
    add_password_parser.add_argument("-w", "--website", action="store")
    add_password_parser.add_argument("-d", "--description", action="store")

    show_password_parser = subparsers.add_parser("show", help="show passwords")
    show_password_parser.add_argument("-n", "--name", action="store", default=None)
    show_password_parser.add_argument("-w", "--website", action="store", default=None)
    show_password_parser.add_argument("-i", "--id", action="store", default=None)

    delete_password_parser = subparsers.add_parser("delete", help="delete password")
    delete_password_parser.add_argument("id")

    delete_user_parser = subparsers.add_parser("delete_user", help="delete user")
    delete_user_parser.add_argument("username")
    delete_user_parser.add_argument("password")

    update_password_parser = subparsers.add_parser("update", help="update password")
    update_password_parser.add_argument("id")
    update_password_parser.add_argument("-p", "--password", default=None)
    update_password_parser.add_argument("-n", "--name", default=None)
    update_password_parser.add_argument("-w", "--website", default=None)
    update_password_parser.add_argument("-d", "--description", default=None)

    args = global_parser.parse_args()

    subcommand = args.subparser_name
    match subcommand:
        case "generate":
            utility_functions.generate_password(args.long, args.query)
        case "evaluate":
            utility_functions.evaluate_password(args.password)
        case "signup":
            user_controller.signup_user(args.username, args.password)
        case "add":
            password_controller.add_password(args.new_password,
                                             args.name,
                                             args.website,
                                             args.description)
        case "show":
            password_controller.select_password(args.name,
                                                args.website,
                                                args.id)
        case "delete":
            password_controller.delete_password(args.id)
        case "login":
            user_controller.login_user(args.username, args.password)
        case "logout":
            user_controller.logout()
        case "user":
            user_controller.display_logged_user()
        case "delete_user":
            user_controller.delete_user(args.username, args.password)
        case "update":
            password_controller.update_password(args.id, args.password, args.name, args.website, args.description)


if __name__ =="__main__":
    main()



import sqlite3
from sqlite3 import Error


def delete_db(path):
    import os
    os.remove(path)


class CreateConnection:
    def __init__(self, path):
        self.connection = None
        self.path = path

    def __enter__(self):
        connection = None
        try:
            connection = sqlite3.connect(self.path)
        except Error as e:
            print(f"The error '{e}' occurred")
        self.connection = connection
        return connection

    def __exit__(self, exc_type, exc_val, exc_tb):
        self.connection.close()


def execute_query(connection, query):
    cursor = connection.cursor()
    try:
        cursor.execute(query)
        connection.commit()
    except Error as e:
        print(f"The error '{e}' occurred (signup to create the database)")


def execute_read_query(connection, query):
    cursor = connection.cursor()
    result = None
    try:
        cursor.execute(query)
        result = cursor.fetchall()
        return result
    except Error as e:
        print(f"The error '{e}' occurred (signup to create the database)")


def create_db(connection):

    create_user_table="""
    CREATE TABLE IF NOT EXISTS user (
      username TEXT PRIMARY KEY,
      password TEXT NOT NULL,
      secret_key TEXT NOT NULL,
      salt TEXT,
      email TEXT
    );
    """
    execute_query(connection, create_user_table)

    create_password_table="""
    CREATE TABLE IF NOT EXISTS password (
      id INTEGER PRIMARY KEY AUTOINCREMENT,
      username TEXT NOT NULL,
      password TEXT NOT NULL,
      password_name TEXT NOT NULL,
      website TEXT,
      description TEXT,
      FOREIGN KEY(username) REFERENCES user(username)
    );
    """
    execute_query(connection, create_password_table)

    create_logged_in_table = """
    CREATE TABLE IF NOT EXISTS loggedIn (
      username PRIMARY KEY,
      key TEXT,
      FOREIGN KEY(username) REFERENCES user(username)
    );
    """
    execute_query(connection, create_logged_in_table)



from tests.test_user_functions import pre_set_db
from pms import user_controller
from pms import password_controller

username = "ahmad"
password = "123"


def signup_and_login_user():
    pre_set_db(user_controller)
    pre_set_db(password_controller)
    user_controller.signup_user(username, password)
    user_controller.login_user(username, password)


def test_add_password(capsys):
    signup_and_login_user()
    password_controller.add_password("1234", "first_p")
    captured = capsys.readouterr()
    assert "password add!" in captured.out


def test_add_password_without_login(capsys):
    pre_set_db()
    user_controller.signup_user(username, password)
    password_controller.add_password("123", "123")
    captured = capsys.readouterr()
    assert "login a user first" in captured.out


def test_show_all_passwords(capsys):
    signup_and_login_user()
    password_controller.add_password("123", "my_pass")
    password_controller.add_password("133", "pass_my")
    password_controller.select_password()
    captured = capsys.readouterr()
    assert f"password:\"{'123'}\"" in captured.out
    assert f"name:{'my_pass'}" in captured.out
    assert f"password:\"{'133'}\"" in captured.out
    assert f"name:{'pass_my'}" in captured.out


def test_show_password_by_name(capsys):
    signup_and_login_user()
    password_controller.add_password("123", "my_pass")
    password_controller.add_password("133", "pass_my")
    password_controller.select_password(name="pass_my")
    captured = capsys.readouterr()
    assert f"password:\"{'123'}\"" not in captured.out
    assert f"name:{'my_pass'}" not in captured.out
    assert f"password:\"{'133'}\"" in captured.out
    assert f"name:{'pass_my'}" in captured.out


def test_show_password_without_add(capsys):
    signup_and_login_user()
    password_controller.select_password()
    captured = capsys.readouterr()
    assert "no passwords matching found!" in captured.out


def test_delete_password(capsys):
    signup_and_login_user()
    password_controller.add_password("123", "my_pass")
    password_controller.select_password()
    captured = capsys.readouterr()
    assert f"password:\"{'123'}\"" in captured.out
    assert f"name:{'my_pass'}" in captured.out
    password_controller.delete_password(password_id="1")
    password_controller.select_password()
    captured = capsys.readouterr()
    assert "no passwords matching found!" in captured.out


def test_delete_same_password_twice(capsys):
    signup_and_login_user()
    password_controller.add_password("123", "my_pass")
    password_controller.delete_password(password_id=1)
    capsys.readouterr()
    password_controller.delete_password(password_id=1)
    captured = capsys.readouterr()
    assert "delete failed: password not found" in captured.out


def test_update_password_valid(capsys):
    signup_and_login_user()
    password_controller.add_password("123", "my_pass")
    password_controller.update_password(password_id="1", new_password="9oo9999")
    captured = capsys.readouterr()
    assert "password updated" in captured.out
    password_controller.select_password(password_id="1")
    captured = capsys.readouterr()
    assert f"password:\"9oo9999\"" in captured.out


def test_update_password_no_argument(capsys):
    signup_and_login_user()
    password_controller.add_password("123", "my_pass")
    password_controller.update_password(password_id="1")
    captured = capsys.readouterr()
    assert "you need to update at least 1 field" in captured.out


def test_update_password_invalid_id(capsys):
    signup_and_login_user()
    password_controller.add_password("123", "my_pass")
    password_controller.update_password(password_id="5")
    captured = capsys.readouterr()
    assert "password id is invalid" in captured.out



import os
from cryptography.fernet import Fernet

from pms.utility_functions import encrypt, decrypt, sha512_hash
from pms import db_functions

PASSWORD_PREFIX = """
YrdL6Uw@ZnN,R%AWzEuc!eCjAQ30,_UWzGD@N.JiH9,$7fiXfHcE5vanpuWNVENlur6%Od8PhC%yYSlaJ$tVQU63Cckk521QX9jysSB!bqK3UA3V5qsVt$Z5
PQZU6js*fx2TUoao%$ZY0ZPKh8r6z@6%FhPTePLDmlOYL!kpUtkwMoWFn5oP7N!z7ZGCh%m!i4nItNsa@IdtNwD,v7eQJy1dkbb0E!o*VbezM!4JvqOOHe.o
0@gPbpn9BmeiGHjZz$S#UPsWNvHq%83PYl6yvG2tJv3Cz4.!G8Ohzpdk,kz."""


class User:
    def __init__(self, username):
        self.username = username
        self.hashed_password = ""
        self.key = None
        self.salt=""

    @staticmethod
    def signup(connection, username, password):
        password = User.password_processing(username, password)
        if User.select_user_from_db(connection, username):
            raise ValueError("username already in db")
        password = password
        user = User(username)
        user.key = Fernet.generate_key()
        user.salt = os.urandom(16)
        encrypted_key = encrypt(str.encode(password), user.salt, user.key)
        encrypted_key = int.from_bytes(encrypted_key, byteorder='big')
        user.salt = int.from_bytes(user.salt, byteorder='big')
        user.hashed_password = sha512_hash(password)
        add_user = f"""
            INSERT INTO user (username, password, secret_key, salt)
             VALUES ("{username}", "{user.hashed_password}", "{encrypted_key}", "{user.salt}")
            """
        db_functions.execute_query(connection, add_user)
        return user

    @staticmethod
    def login(connection, username, password):
        if User.get_logged_user(connection) is not None:
            raise ValueError("a user is already logged in")
        password = User.password_processing(username, password)
        user_data = User.select_user_from_db(connection, username)
        if not user_data:
            raise ValueError("username or password incorrect")
        user_data = user_data[0]
        password = password
        hashed_password = sha512_hash(password)
        if hashed_password != user_data[1]:
            raise ValueError("username or password incorrect")
        user = User(username)
        user.hashed_password = hashed_password
        salt = int(user_data[3])
        salt= salt.to_bytes(length=16, byteorder='big')
        encrypted_key = int(user_data[2])
        encrypted_key = encrypted_key.to_bytes(length=140, byteorder='big')
        user.key = decrypt(str.encode(password), salt, encrypted_key)
        db_functions.execute_query(connection, f"""
                    INSERT INTO loggedIn (username, key)
                     VALUES ("{user.username}", "{user.key.decode()}")
                    """)
        return user

    @staticmethod
    def select_user_from_db(connection, username):
        user_data = db_functions.execute_read_query(connection, f"""
                    SELECT * FROM user WHERE username="{username}";
                    """)
        return user_data

    @staticmethod
    def password_processing(username, password):
        return PASSWORD_PREFIX+password+username

    @staticmethod
    def get_logged_user(connection):
        user_data= db_functions.execute_read_query(connection, "SELECT * FROM loggedIn")
        if not user_data:
            return None
        user_data = user_data[0]
        user = User(user_data[0])
        user.key = user_data[1].encode()
        return user

    @staticmethod
    def logout(connection):
        if User.get_logged_user(connection) is not None:
            db_functions.execute_query(connection, "DELETE FROM loggedIn")
            return True
        return False

    @staticmethod
    def delete(connection, username, password):
        password = User.password_processing(username, password)
        user_data = User.select_user_from_db(connection, username)
        if not user_data:
            raise ValueError("username or password incorrect")
        user_data = user_data[0]
        password = password
        hashed_password = sha512_hash(password)
        if hashed_password != user_data[1]:
            raise ValueError("username or password incorrect")
        db_functions.execute_query(connection, f"DELETE FROM password WHERE username = \"{username}\"")
        db_functions.execute_query(connection, f"DELETE FROM loggedIn WHERE username = \"{username}\"")
        db_functions.execute_query(connection, f"DELETE FROM user WHERE username = \"{username}\"")


class Password:
    def __init__(self):
        self.id = None
        self.username = None
        self.password = None
        self.name = None
        self.website = None
        self.description = None

    @staticmethod
    def add_password(connection, user, password, name, website="", description=""):
        if user.key is None:
            raise ValueError("user isn't login")
        if website is None:
            website = ""
        if description is None:
            description = ""
        f = Fernet(user.key)
        password = f.encrypt(password.encode()).decode()
        add_password = f"""
                    INSERT INTO password (username, password, password_name, website, description)
                     VALUES ("{user.username}", "{password}", "{name}", "{website}", "{description}")
                    """
        db_functions.execute_query(connection, add_password)

    @staticmethod
    def db_data_to_list(password_data, key):
        password_list = []
        for item in password_data:
            password = Password()
            password.id = item[0]
            password.username = item[1]
            f = Fernet(key)
            password.password = f.decrypt(item[2].encode()).decode()
            password.name = item[3]
            password.website = item[4]
            password.description = item[5]
            password_list.append(password)
        return password_list

    @staticmethod
    def select_password(connection, user, name=None, website=None, password_id=None):
        if user.key is None:
            raise ValueError("user isn't login")
        condition =""
        if name is not None:
            condition = f' AND password_name="{name}"'
        elif website is not None:
            condition = f' AND website="{website}"'
        elif password_id is not None:
            condition = f' AND id="{password_id}"'
        password_data = db_functions.execute_read_query(connection, f"""
                                SELECT * FROM password WHERE username="{user.username}"{condition};
                                """)
        if not password_data:
            return None
        return Password.db_data_to_list(password_data, user.key)

    @staticmethod
    def delete_password(connection, user, password_id):
        password = Password.select_password(connection, user, password_id=password_id)
        if password is None:
            raise ValueError("delete failed: password not found")
        db_functions.execute_query(connection, f"DELETE FROM password WHERE id = \"{password_id}\"")

    def __str__(self):
        my_str = f"id:{self.id} "
        my_str += f"password:\"{self.password}\" "
        my_str += f"name:{self.name} "
        my_str += f"website:{self.website} "
        my_str += f"description:{self.description}"
        return my_str

    @staticmethod
    def update_password(connection, user, password_id, new_password=None, new_name=None, new_website=None,
                        new_description=None):
        password = Password.select_password(connection, user, password_id=password_id)
        if password is None:
            raise ValueError("password id is invalid")
        field_list=[]
        if new_password is not None:
            f = Fernet(user.key)
            new_password = f.encrypt(new_password.encode()).decode()
            field_list.append(f"password=\"{new_password}\"")
        if new_name is not None:
            field_list.append(f"password_name=\"{new_name}\"")
        if new_website is not None:
            field_list.append(f"website=\"{new_website}\"")
        if new_description is not None:
            field_list.append(f"description=\"{new_description}\"")
        if not field_list:
            raise ValueError("you need to update at least 1 field")
        query = f"""
        UPDATE password SET
        {",".join(field_list)}
        WHERE id = \"{password_id}\"        
        """
        db_functions.execute_query(connection, query)



from pms import db_functions
from pms.user_controller import db_path, get_logged_user
from pms.models import Password


def add_password(new_password, name, website="", description=""):
    with db_functions.CreateConnection(db_path) as connection:
        try:
            user = get_logged_user(connection)
            Password.add_password(connection, user, new_password, name, website, description)
            print("password add!")
        except ValueError as v:
            print(v.__str__())


def select_password(name=None, password_website=None, password_id=None):
    with db_functions.CreateConnection(db_path) as connection:
        try:
            user = get_logged_user(connection)
            password_list = Password.select_password(connection, user, name, password_website, password_id)
            if password_list is None:
                print("no passwords matching found!")
            else:
                for item in password_list:
                    print(item)
        except ValueError as v:
            print(v.__str__())


def delete_password(password_id):
    with db_functions.CreateConnection(db_path) as connection:
        try:
            user = get_logged_user(connection)
            Password.delete_password(connection, user, password_id)
            print("password deleted")
        except ValueError as v:
            print(v.__str__())


def update_password(password_id, new_password=None, new_name=None, new_website=None, new_description=None):
    with db_functions.CreateConnection(db_path) as connection:
        try:
            user = get_logged_user(connection)
            Password.update_password(connection, user, password_id, new_password, new_name, new_website, new_description)
            print("password updated")
        except ValueError as v:
            print(v.__str__())


import re

import pytest
from pms.utility_functions import evaluate_password, generate_password


def test_generate_single_query(capsys):
    generate_password(5, ["a"])
    assert "aaaaa" in capsys.readouterr().out


def test_generate_single_query2(capsys):
    generate_password(5, ["abc"])
    assert re.search(r"[abc]{5}", capsys.readouterr().out)


def test_generate_multiple_queries(capsys):
    generate_password(10, ["a", "b", "c"])
    captured = capsys.readouterr()
    assert re.search(r"[abc]{10}", captured.out)
    assert re.search(r"a", captured.out)
    assert re.search(r"b", captured.out)
    assert re.search(r"c", captured.out)
    assert not re.search(r"d", captured.out)


def test_generate_too_low_length():
    try:
        generate_password(4, ["a", "b", "c", "d", "e"])
        assert False
    except ValueError:
        assert True
    except Exception:
        assert False


@pytest.mark.parametrize("password, expected_result", [
    ("1aB!1aB!", 5),
    ("&ab1&Ab1", 5),
    ("-ab452GF", 5),
    ("abc!@#123", 4),
    ("1111111111", 2),
    ("11111", 1),
    ("aB!", 0)
])
def test_evaluate_passwords(password, expected_result, capsys):
    evaluate_password(password)
    assert str(expected_result) in capsys.readouterr().out




from pms import db_functions
from pms.models import User
import pathlib

db_path = pathlib.Path.joinpath(pathlib.Path(__file__).parent, "sql.db")


def signup_user(username, password):
    with db_functions.CreateConnection(db_path) as connection:
        db_functions.create_db(connection)
        try:
            user = User.signup(connection, username, password)
            print("user add!")
            return user
        except ValueError as v:
            print(v.__str__())


def login_user(username, password):
    with db_functions.CreateConnection(db_path) as connection:
        try:
            User.login(connection, username, password)
            print("user logged in")
        except ValueError as v:
            print(v.__str__())


def get_logged_user(connection):
    user = User.get_logged_user(connection)
    if user is None:
        raise ValueError("login a user first")
    return user


def logout():
    with db_functions.CreateConnection(db_path) as connection:
        if User.logout(connection):
            print("user logged out")
        else:
            print("user not login!")


def display_logged_user():
    with db_functions.CreateConnection(db_path) as connection:
        try:
            user = get_logged_user(connection)
            print(f"logged user is:{user.username}")
        except ValueError:
            print("no user logged")


def delete_user(username, password):
    with db_functions.CreateConnection(db_path) as connection:
        try:
            User.delete(connection, username, password)
            print("user deleted")
        except ValueError as v:
            print(v.__str__())



import random
import re
import string

import base64
import hashlib
from cryptography.fernet import Fernet
from cryptography.hazmat.primitives import hashes
from cryptography.hazmat.primitives.kdf.pbkdf2 import PBKDF2HMAC

UPPER_CASE = string.ascii_uppercase
LOWER_CASE = string.ascii_lowercase
NUMBERS = string.digits
SPECIAL_CHAR = "!@#$%*,._"
#      & - have special meaning in cli, so they aren't included in the query


def generate_password(length=16, char_query=None):
    if char_query is None:
        char_query = [UPPER_CASE, LOWER_CASE, NUMBERS, SPECIAL_CHAR]

    if len(char_query) > length:
        raise ValueError("char query can't be longer than the password!")
    my_password=[]
    # use at least 1 char from every query
    for item in char_query:
        index = random.randint(0, len(my_password))
        my_password[index:index] = random.choice(item)
    char_range = "".join(char_query)  # a string of all queries combined to generate the rest of the password
    while len(my_password)<length:
        index = random.randint(0, len(my_password))
        my_password[index:index] = random.choice(char_range)

    print("".join(my_password))


def evaluate_password(password):
    strength = 5
    tips_list=[]
    if len(password)<4:
        print("password strength is 0 from 5")
        print("password is too short to even consider as password!")
    elif len(password)<8:
        tips_list.append("password is short, add more symbols to it")
        strength -= 1

    if re.search(r"\d", password) is None:
        tips_list.append("add numbers")
        strength -= 1

    if re.search(r"[A-Z]", password) is None:
        tips_list.append("add uppercase letters")
        strength -= 1

    if re.search(r"[a-z]", password) is None:
        tips_list.append("add lowercase letters")
        strength -= 1

    if re.search('['+SPECIAL_CHAR+"&-"+']', password) is None:
        tips_list.append("add special characters")
        strength -= 1

    result = (strength, "\n".join(tips_list))
    print(f"password strength is {result[0]} from 5")
    if result[1]:
        print(f"tips to make your password stronger:\n{result[1]}")


def encrypt(password, salt, data):
    kdf = PBKDF2HMAC(
        algorithm=hashes.SHA256(),
        length=32,
        salt=salt,
        iterations=480000,
    )
    key = base64.urlsafe_b64encode(kdf.derive(password))
    f = Fernet(key)
    return f.encrypt(data)


def decrypt(password, salt, data):
    kdf = PBKDF2HMAC(
        algorithm=hashes.SHA256(),
        length=32,
        salt=salt,
        iterations=480000,
    )
    key = base64.urlsafe_b64encode(kdf.derive(password))
    f = Fernet(key)
    return f.decrypt(data)


def sha512_hash(password):
    return hashlib.sha512(password.encode('utf-8')).hexdigest()

